@@ -1,6 +1,5 @@
-/*	$Id$ */
 /*
- * Copyright (c) 2017--2021 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -768,7 +767,7 @@ node_clonetree(const struct lowdown_node *v, size_t *id)
 	struct lowdown_node *n, *nn;
 	const struct lowdown_node *vv;
 
-	if ((n = node_clone(v, *id++)) == NULL)
+	if ((n = node_clone(v, (*id)++)) == NULL)
 		return NULL;
 
 	TAILQ_FOREACH(vv, &v->children, entries) {
