@@ -55,6 +55,7 @@ struct 	nroff {
 	const char		 *cb; /* fixed-width bold font */
 	const char		 *ci; /* fixed-width italic font */
 	const char		 *cbi; /* fixed-width bold-italic font */
+	const char		 *templ; /* output template */
 };
 
 enum	bscope {
@@ -93,106 +94,6 @@ struct	bnode {
 
 TAILQ_HEAD(bnodeq, bnode);
 
-/*
- * Escape unsafe text into roff output such that no roff features are
- * invoked by the text (macros, escapes, etc.).
- * If "oneline" is non-zero, newlines are replaced with spaces.
- * If "literal", doesn't strip leading space.
- * Return zero on failure, non-zero on success.
- */
-static int
-hesc_nroff(struct lowdown_buf *ob, const char *data, size_t size,
-    int oneline, int literal, int esc)
-{
-	size_t	 	i = 0;
-	unsigned char	ch;
-
-	if (size == 0)
-		return 1;
-
-	if (!esc && oneline) {
-		assert(!literal);
-		for (i = 0; i < size; i++) {
-			ch = data[i] == '\n' ? ' ' : data[i];
-			if (!hbuf_putc(ob, ch))
-				return 0;
-			if (ch != ' ')
-				continue;
-			while (i < size && isspace((unsigned char)data[i]))
-				i++;
-			i--;
-		}
-		return 1;
-	} else if (!esc)
-		return hbuf_put(ob, data, size);
-
-	/* Strip leading whitespace. */
-
-	if (!literal && ob->size > 0 && ob->data[ob->size - 1] == '\n')
-		while (i < size && (data[i] == ' ' || data[i] == '\n'))
-			i++;
-
-	/*
-	 * According to mandoc_char(7), we need to escape the backtick,
-	 * single apostrophe, and tilde or else they'll be considered as
-	 * special Unicode output.
-	 * Slashes need to be escaped too.
-	 * We also escape double-quotes because this text might be used
-	 * within quoted macro arguments.
-	 */
-
-	for ( ; i < size; i++)
-		switch (data[i]) {
-		case '^':
-			if (!HBUF_PUTSL(ob, "\\(ha"))
-				return 0;
-			break;
-		case '~':
-			if (!HBUF_PUTSL(ob, "\\(ti"))
-				return 0;
-			break;
-		case '`':
-			if (!HBUF_PUTSL(ob, "\\(ga"))
-				return 0;
-			break;
-		case '"':
-			if (!HBUF_PUTSL(ob, "\\(dq"))
-				return 0;
-			break;
-		case '\n':
-			if (!hbuf_putc(ob, oneline ? ' ' : '\n'))
-				return 0;
-			if (literal)
-				break;
-
-			/* Prevent leading spaces on the line. */
-
-			while (i + 1 < size && 
-			       (data[i + 1] == ' ' || 
-				data[i + 1] == '\n'))
-				i++;
-			break;
-		case '\\':
-			if (!HBUF_PUTSL(ob, "\\e"))
-				return 0;
-			break;
-		case '\'':
-		case '.':
-			if (!oneline &&
-			    ob->size > 0 && 
-			    ob->data[ob->size - 1] == '\n' &&
-			    !HBUF_PUTSL(ob, "\\&"))
-				return 0;
-			/* FALLTHROUGH */
-		default:
-			if (!hbuf_putc(ob, data[i]))
-				return 0;
-			break;
-		}
-
-	return 1;
-}
-
 static const char *
 nstate_colour_buf(unsigned int ft)
 {
@@ -480,12 +381,12 @@ bqueue_flush(const struct nroff *st, struct lowdown_buf *ob,
 
 		if (bn->scope == BSCOPE_LITERAL) {
 			assert(bn->buf != NULL);
-			if (!hesc_nroff(ob, bn->buf,
-			    strlen(bn->buf), 0, 1, 1))
+			if (!lowdown_nroff_esc(ob, bn->buf,
+			    strlen(bn->buf), 0, 1))
 				return 0;
 		} else if (bn->buf != NULL)
-			if (!hesc_nroff(ob, bn->buf,
-			    strlen(bn->buf), 0, 0, 1))
+			if (!lowdown_nroff_esc(ob, bn->buf,
+			    strlen(bn->buf), 0, 0))
 				return 0;
 
 		/*
@@ -539,8 +440,8 @@ bqueue_flush(const struct nroff *st, struct lowdown_buf *ob,
 				bn->scope == BSCOPE_SEMI);
 			if (!hbuf_putc(ob, ' '))
 				return 0;
-			if (!hesc_nroff(ob, bn->args,
-			    strlen(bn->args), 1, 0, 1))
+			if (!lowdown_nroff_esc(ob, bn->args,
+			    strlen(bn->args), 1, 0))
 				return 0;
 		}
 
@@ -582,7 +483,7 @@ hbuf2shortlink(const struct lowdown_buf *link)
 		goto out;
 	if (!hbuf_shortlink(tmp, link))
 		goto out;
-	if (!hesc_nroff(slink, tmp->data, tmp->size, 1, 0, 1))
+	if (!lowdown_nroff_esc(slink, tmp->data, tmp->size, 1, 0))
 		goto out;
 	ret = strndup(slink->data, slink->size);
 out:
@@ -1061,7 +962,8 @@ rndr_header(struct nroff *st, struct bnodeq *obq, struct bnodeq *bq,
 		 * escaping into roff.
 		 */
 
-		bn->args = strndup(buf->data, buf->size);
+		bn->args = buf->size == 0 ?
+			strdup("") : strndup(buf->data, buf->size);
 		if (bn->args == NULL)
 			goto out;
 
@@ -1738,8 +1640,8 @@ rndr_meta(struct nroff *st, const struct lowdown_node *n,
 }
 
 static int
-rndr_doc_header(struct nroff *st, struct bnodeq *obq,
-    const struct lowdown_metaq *mq)
+rndr_root(struct nroff *st, struct bnodeq *obq,
+    struct bnodeq *bq, const struct lowdown_metaq *mq)
 {
 	struct lowdown_buf		*ob = NULL;
 	struct bnode			*bn;
@@ -1753,8 +1655,13 @@ rndr_doc_header(struct nroff *st, struct bnodeq *obq,
 					*msheader = NULL,
 					*manheader = NULL;
 
-	if (!(st->flags & LOWDOWN_STANDALONE))
+	if (!(st->flags & LOWDOWN_STANDALONE)) {
+		TAILQ_CONCAT(obq, bq, entries);
 		return 1;
+	} else if (st->templ != NULL) {
+		TAILQ_CONCAT(obq, bq, entries);
+		return 1;
+	}
 
 	TAILQ_FOREACH(m, mq, entries)
 		if (strcasecmp(m->key, "author") == 0)
@@ -1866,11 +1773,11 @@ rndr_doc_header(struct nroff *st, struct bnodeq *obq,
 		if (!HBUF_PUTSL(ob, "\""))
 			goto out;
 		if (title != NULL &&
-		    !hesc_nroff(ob, title, strlen(title), 1, 0, 1))
+		    !lowdown_nroff_esc(ob, title, strlen(title), 1, 0))
 			goto out;
 		if (!HBUF_PUTSL(ob, "\" \""))
 			goto out;
-		if (!hesc_nroff(ob, sec, strlen(sec), 1, 0, 1))
+		if (!lowdown_nroff_esc(ob, sec, strlen(sec), 1, 0))
 			goto out;
 		if (!HBUF_PUTSL(ob, "\" \""))
 			goto out;
@@ -1881,7 +1788,7 @@ rndr_doc_header(struct nroff *st, struct bnodeq *obq,
 		 */
 
 		if (date != NULL &&
-		    !hesc_nroff(ob, date, strlen(date), 1, 0, 1))
+		    !lowdown_nroff_esc(ob, date, strlen(date), 1, 0))
 			goto out;
 		if (!HBUF_PUTSL(ob, "\""))
 			goto out;
@@ -1895,15 +1802,15 @@ rndr_doc_header(struct nroff *st, struct bnodeq *obq,
 		if (source != NULL || volume != NULL) {
 			if (!HBUF_PUTSL(ob, " \""))
 				goto out;
-			if (source != NULL && !hesc_nroff
-			    (ob, source, strlen(source), 1, 0, 1))
+			if (source != NULL && !lowdown_nroff_esc
+			    (ob, source, strlen(source), 1, 0))
 				goto out;
 			if (!HBUF_PUTSL(ob, "\""))
 				goto out;
 			if (!HBUF_PUTSL(ob, " \""))
 				goto out;
-			if (volume != NULL && !hesc_nroff
-			    (ob, volume, strlen(volume), 1, 0, 1))
+			if (volume != NULL && !lowdown_nroff_esc
+			    (ob, volume, strlen(volume), 1, 0))
 				goto out;
 			if (!HBUF_PUTSL(ob, "\""))
 				goto out;
@@ -1914,6 +1821,7 @@ rndr_doc_header(struct nroff *st, struct bnodeq *obq,
 
 	rc = 1;
 out:
+	TAILQ_CONCAT(obq, bq, entries);
 	hbuf_free(ob);
 	return rc;
 }
@@ -2002,7 +1910,6 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 		rc = rndr_definition_title(st, obq, &tmpbq);
 		break;
 	case LOWDOWN_DOC_HEADER:
-		rc = rndr_doc_header(st, obq, mq);
 		break;
 	case LOWDOWN_META:
 		if (n->chng != LOWDOWN_CHNG_DELETE)
@@ -2038,9 +1945,9 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 		rc = rndr_table_cell(obq, &tmpbq, &n->rndr_table_cell);
 		break;
 	case LOWDOWN_ROOT:
-		TAILQ_CONCAT(obq, &tmpbq, entries);
 		assert(st->footdepth == 0);
-		rc = rndr_footnotes(st, obq, 1);
+		rc = rndr_footnotes(st, &tmpbq, 1) &&
+			rndr_root(st, obq, &tmpbq, mq);
 		break;
 	case LOWDOWN_BLOCKHTML:
 		rc = rndr_raw_block(st, obq, &n->rndr_blockhtml);
@@ -2162,6 +2069,7 @@ lowdown_nroff_rndr(struct lowdown_buf *ob,
 	void *arg, const struct lowdown_node *n)
 {
 	struct nroff		*st = arg;
+	struct lowdown_buf	*tmp = NULL;
 	struct lowdown_metaq	 metaq;
 	int			 rc = 0;
 	struct bnodeq		 bq;
@@ -2176,12 +2084,15 @@ lowdown_nroff_rndr(struct lowdown_buf *ob,
 	st->post_para = 0;
 
 	if (rndr(&metaq, st, n, &bq)) {
-		if (!bqueue_flush(st, ob, &bq))
+		if ((tmp = hbuf_new(64)) == NULL)
 			goto out;
-		if (ob->size && ob->data[ob->size - 1] != '\n' &&
-		    !hbuf_putc(ob, '\n'))
+		if (!bqueue_flush(st, tmp, &bq))
 			goto out;
-		rc = 1;
+		if (tmp->size && tmp->data[tmp->size - 1] != '\n' &&
+		    !hbuf_putc(tmp, '\n'))
+			goto out;
+		rc = st->templ == NULL ? hbuf_putb(ob, tmp) :
+			lowdown_template(st->templ, tmp, ob, &metaq, 0);
 	}
 
 out:
@@ -2190,6 +2101,7 @@ out:
 		free(st->foots[i]);
 	}
 
+	hbuf_free(tmp);
 	free(st->foots);
 	st->footsz = st->footpos = 0;
 	st->foots = NULL;
@@ -2213,6 +2125,7 @@ lowdown_nroff_new(const struct lowdown_opts *opts)
 	p->cb = opts != NULL ? opts->nroff.cb : NULL;
 	p->ci = opts != NULL ? opts->nroff.ci : NULL;
 	p->cbi = opts != NULL ? opts->nroff.cbi : NULL;
+	p->templ = opts != NULL ? opts->templ : NULL;
 
 	/*
 	 * Set the default "constant width" fonts.  This is complicated
