@@ -43,8 +43,9 @@ struct term {
 	struct tstack		 *stack; /* stack of nodes */
 	size_t			  stackmax; /* size of stack */
 	size_t			  stackpos; /* position in stack */
-	size_t			  maxcol; /* soft limit */
+	size_t			  width; /* soft width of content */
 	size_t			  hmargin; /* left of content */
+	size_t			  hpadding; /* left of content */
 	size_t			  vmargin; /* before/after content */
 	struct lowdown_buf	 *tmp; /* for temporary allocations */
 	wchar_t			 *buf; /* buffer for counting wchar */
@@ -172,6 +173,29 @@ rndr_mbswidth(struct term *term, const char *buf, size_t sz)
 	return csz == (size_t)-1 ? sz : csz;
 }
 
+/*
+ * Determine whether a link URL is relative.  Use a simple heuristic to
+ * accomplish this: a relative URL is one without a schema.  Returns
+ * zero if not a relative link, non-zero if it is.
+ */
+static int
+link_isrelative(const struct lowdown_buf *link)
+{
+	const char	*colon;
+	size_t	 	 rem;
+
+	/* If there's no colon, it's a relative link (no schema) */
+
+	if ((colon = memchr(link->data, ':', link->size)) == NULL)
+		return 1;
+
+	/* If there's a slash before the colon, it's a (rel) path. */
+
+	assert(colon > link->data);
+	rem = colon - link->data;
+	return memchr(link->data, '/', rem) != NULL;
+}
+
 /*
  * Copy the buffer into "out", escaping along the width.
  * Returns the number of actual printed columns, which in the case of
@@ -248,7 +272,7 @@ rndr_buf_unstyle(const struct term *term,
 }
 
 /*
- * Start sequence for a terminal link (only in ANSI mode).  
+ * Start sequence for a terminal link (only in ANSI mode).
  */
 static int
 rndr_buf_osc8_open(const struct term *term, struct lowdown_buf *out,
@@ -258,7 +282,7 @@ rndr_buf_osc8_open(const struct term *term, struct lowdown_buf *out,
 
 	if (term->opts & LOWDOWN_TERM_NOANSI)
 		return 1;
-	
+
 	if (n->type == LOWDOWN_LINK_AUTO)
 		uri = &n->rndr_autolink.link;
 	else if (n->type == LOWDOWN_LINK)
@@ -469,12 +493,12 @@ rndr_buf_endwords(struct term *term, struct lowdown_buf *out,
 	 * matter where this appears in relation to other styling).
 	 */
 
+	if (rndr_buf_endstyle(n) && !rndr_buf_unstyle(term, out, NULL))
+        	return 0;
+	if (osty != NULL && !rndr_buf_unstyle(term, out, osty))
+        	return 0;
 	if (term->in_link && !rndr_buf_osc8_close(term, out))
-		return 0;
-	if (rndr_buf_endstyle(n))
-		return rndr_buf_unstyle(term, out, NULL);
-	if (osty != NULL)
-		return rndr_buf_unstyle(term, out, osty);
+        	return 0;
 	return 1;
 }
 
@@ -581,21 +605,6 @@ rndr_buf_startline_prefixes(struct term *term,
 
 	emit = term->stack[i].lines++;
 
-	/*
-	 * If we're below the document root and not a header, that means
-	 * we're in a body part.  Emit the general body indentation.
-	 */
-
-	if (*depth == 0 && n->type != LOWDOWN_HEADER) {
-		if (!hbuf_puts(out, pfx_body.text))
-			return 0;
-		rndr_buf_advance(term, pfx_body.cols);
-	} else if (*depth == 0) {
-		if (!hbuf_puts(out, pfx_header.text))
-			return 0;
-		rndr_buf_advance(term, pfx_header.cols);
-	}
-
 	/*
 	 * Output any prefixes.
 	 * Any output must have rndr_buf_style() and set pstyle so that
@@ -619,6 +628,9 @@ rndr_buf_startline_prefixes(struct term *term,
 		for (i = 0; i < term->hmargin; i++)
 			if (!HBUF_PUTSL(out, " "))
 				return 0;
+		for (i = 0; i < term->hpadding; i++)
+			if (!HBUF_PUTSL(out, " "))
+				return 0;
 		break;
 	case LOWDOWN_BLOCKQUOTE:
 		rndr_node_style_apply(&sinner, &sty_bkqt_pfx);
@@ -862,8 +874,8 @@ rndr_buf_startwords(struct term *term, struct lowdown_buf *out,
  */
 static int
 rndr_buf_literal(struct term *term, struct lowdown_buf *out,
-	const struct lowdown_node *n, const struct lowdown_buf *in,
-	const struct sty *osty)
+    const struct lowdown_node *n, const struct lowdown_buf *in,
+    const struct sty *osty)
 {
 	size_t		 i = 0, len;
 	const char	*start;
@@ -902,9 +914,10 @@ rndr_buf(struct term *term, struct lowdown_buf *out,
 	const struct lowdown_node *n, const struct lowdown_buf *in,
 	const struct sty *osty)
 {
-	size_t				 i = 0, len, cols;
+	size_t				 i = 0, len, cols, nlen;
 	ssize_t				 ret;
-	int				 needspace, begin = 1, end = 0;
+	int				 needspace, hasspace,
+					 begin = 1, end = 0;
 	const char			*start;
 	const struct lowdown_node	*nn;
 
@@ -913,22 +926,30 @@ rndr_buf(struct term *term, struct lowdown_buf *out,
 	  	    nn->type == LOWDOWN_BLOCKHTML)
 			return rndr_buf_literal(term, out, n, in, osty);
 
-	/* Start each word by seeing if it has leading space. */
-
 	while (i < in->size) {
+		/*
+		 * Whether we need a space (word begins with space) and
+		 * have a space (current printed content ends with one).
+		 */
+
 		needspace = isspace((unsigned char)in->data[i]);
+		hasspace = out->size > 0 &&
+			isspace((unsigned char)out->data[out->size - 1]);
+
+		/* Skip to next word, then see how long the word is. */
 
 		while (i < in->size &&
 		       isspace((unsigned char)in->data[i]))
 			i++;
-
-		/* See how long it the coming word (may be 0). */
-
 		start = &in->data[i];
 		while (i < in->size &&
 		       !isspace((unsigned char)in->data[i]))
 			i++;
+
+		/* Get length and adjusted length (includes space). */
+
 		len = &in->data[i] - start;
+		nlen = len + (needspace ? 1 : 0);
 
 		/*
 		 * If we cross our maximum width and are preceded by a
@@ -939,10 +960,9 @@ rndr_buf(struct term *term, struct lowdown_buf *out,
 		 * This will also unset the current style.
 		 */
 
-		if ((needspace ||
-	 	     (out->size && isspace
-		      ((unsigned char)out->data[out->size - 1]))) &&
-		    term->col && term->col + len > term->maxcol) {
+		if ((needspace || hasspace) &&
+		    term->col > 0 &&
+		    term->col + nlen >= term->width) {
 			if (!rndr_buf_endline(term, out, n, osty))
 				return 0;
 			end = 0;
@@ -1029,6 +1049,32 @@ rndr_entity(struct lowdown_buf *buf, int32_t val)
 	return 1;
 }
 
+/*
+ * Render a horizontal rule by repeating the character(s) in "hr" until
+ * the full screen width has been met.  This presumes that the caller
+ * has started a new line.  The style may be NULL.  A zero-length rule
+ * is silently ignored.
+ */
+static int
+rndr_hrule(struct term *st, struct lowdown_buf *ob, const char *hr,
+    const struct lowdown_node *n, const struct sty *sty)
+{
+	ssize_t	 ssz;
+	size_t	 sz, i;
+
+	if ((sz = strlen(hr)) == 0)
+		return 1;
+	if ((ssz = rndr_mbswidth(st, hr, sz)) < 0)
+		return 0;
+	if (ssz == 0)
+		return 1;
+	hbuf_truncate(st->tmp);
+	for (i = 0; i + ssz <= st->width; i += ssz)
+		if (!hbuf_puts(st->tmp, hr))
+			return 0;
+	return rndr_buf_literal(st, ob, n, st->tmp, sty);
+}
+
 /*
  * Adjust the stack of current nodes we're looking at.
  */
@@ -1104,12 +1150,12 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 				 * line wrapping.
 				 */
 
-				maxcol = st->maxcol;
+				maxcol = st->width;
 				last_blank = st->last_blank;
 				col = st->col;
 
 				st->last_blank = 0;
-				st->maxcol = SIZE_MAX;
+				st->width = SIZE_MAX;
 				st->col = 1;
 				if (!rndr(celltmp, st, cell))
 					goto out;
@@ -1117,7 +1163,7 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 					widths[i] = st->col;
 				st->last_blank = last_blank;
 				st->col = col;
-				st->maxcol = maxcol;
+				st->width = maxcol;
 			}
 	}
 
@@ -1137,12 +1183,12 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 			TAILQ_FOREACH(cell, &row->children, entries) {
 				i = cell->rndr_table_cell.col;
 				hbuf_truncate(celltmp);
-				maxcol = st->maxcol;
+				maxcol = st->width;
 				last_blank = st->last_blank;
 				col = st->col;
 
 				st->last_blank = 0;
-				st->maxcol = SIZE_MAX;
+				st->width = SIZE_MAX;
 				st->col = 1;
 				if (!rndr(celltmp, st, cell))
 					goto out;
@@ -1184,14 +1230,16 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 
 				st->last_blank = last_blank;
 				st->col = col;
-				st->maxcol = maxcol;
+				st->width = maxcol;
 
 				if (TAILQ_NEXT(cell, entries) == NULL)
 					continue;
 
-				if (!rndr_buf_style(st, rowtmp, &sty_table) ||
-				    !hbuf_printf(rowtmp, " %s ", ifx_table_col) ||
-				    !rndr_buf_unstyle(st, rowtmp, &sty_table))
+				if (!HBUF_PUTSL(rowtmp, " ") ||
+				    !rndr_buf_style(st, rowtmp, &sty_tbl) ||
+				    !hbuf_puts(rowtmp, ifx_tbl_col) ||
+				    !rndr_buf_unstyle(st, rowtmp, &sty_tbl) ||
+				    !HBUF_PUTSL(rowtmp, " "))
 					goto out;
 			}
 
@@ -1224,19 +1272,41 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 			st->stackpos++;
 			if (!rndr_stackpos_init(st, n))
 				goto out;
-			if (!rndr_buf_startline(st, ob, n, &sty_table))
+			if (!rndr_buf_startline(st, ob, n, &sty_tbl))
 				goto out;
+
+			/*
+			 * Output the row line.  This consists of:
+			 *
+			 *   inter    padding
+			 *       |    | |
+			 *       |    | |
+			 *   ----+-----+-----
+			 *   xyz   xyz   xyz
+			 *   |     |
+			 *   content
+			 *
+			 * So starting with the content, ending with a
+			 * padding of one byte (encompassed in the
+			 * width), the inter mark or nothing if at the
+			 * end, then another padding or nothing if at
+			 * the end.
+			 */
 			for (i = 0; i < n->rndr_table.columns; i++) {
+				/* Pre-padding. */
+				if (i > 0 && !hbuf_puts(ob, ifx_tbl_row))
+					goto out;
+				/* Content and post-padding. */
 				for (j = 0; j < widths[i]; j++)
-					if (!hbuf_puts(ob, ifx_table_row))
+					if (!hbuf_puts(ob, ifx_tbl_row))
 						goto out;
+				/* Inter. */
 				if (i < n->rndr_table.columns - 1 &&
-				    !hbuf_printf(ob, "%s%s",
-				    ifx_table_col, ifx_table_row))
+				    !hbuf_puts(ob, ifx_tbl_mcol))
 					goto out;
 			}
 			rndr_buf_advance(st, 1);
-			if (!rndr_buf_endline(st, ob, n, &sty_table))
+			if (!rndr_buf_endline(st, ob, n, &sty_tbl))
 				goto out;
 			if (!rndr_buf_vspace(st, ob, n, 1))
 				goto out;
@@ -1406,10 +1476,12 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		vs = n->rndr_math.blockmode ? 1 : 0;
 		break;
 	case LOWDOWN_DEFINITION_DATA:
-	case LOWDOWN_HRULE:
 	case LOWDOWN_LINEBREAK:
 		vs = 1;
 		break;
+	case LOWDOWN_HRULE:
+		vs = 2;
+		break;
 	case LOWDOWN_LISTITEM:
 		vs = 1;
 		if (n->rndr_listitem.flags & HLIST_FL_BLOCK) {
@@ -1500,10 +1572,7 @@ rndr(struct lowdown_buf *ob, struct term *st,
 			return 0;
 		break;
 	case LOWDOWN_HRULE:
-		hbuf_truncate(st->tmp);
-		if (!hbuf_puts(st->tmp, ifx_hrule))
-			return 0;
-		if (!rndr_buf(st, ob, n, st->tmp, NULL))
+		if (!rndr_hrule(st, ob, ifx_hrule, n, NULL))
 			return 0;
 		break;
 	case LOWDOWN_FOOTNOTE:
@@ -1531,7 +1600,7 @@ rndr(struct lowdown_buf *ob, struct term *st,
 			if (!rndr_buf(st, ob, n, st->tmp, NULL))
 				return 0;
 		} else {
-			if (!rndr_buf(st, ob, n, 
+			if (!rndr_buf(st, ob, n,
 			     &n->rndr_entity.text, &sty_bad_ent))
 				return 0;
 		}
@@ -1563,12 +1632,27 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		}
 		break;
 	case LOWDOWN_LINK:
-		if (st->opts & LOWDOWN_TERM_NOLINK)
+		/*
+		 * The child content of the link has already been
+		 * produced to the output buffer.  Inhibit printing the
+		 * link address if requested for all links or if a
+		 * relative address and requested only for those.
+		 */
+
+		if ((st->opts & LOWDOWN_TERM_NOLINK) ||
+		    ((st->opts & LOWDOWN_TERM_NORELLINK) &&
+		     link_isrelative(&n->rndr_link.link)))
 			break;
+
+		/* Separate between text and link address. */
+
 		hbuf_truncate(st->tmp);
-		if (!HBUF_PUTSL(st->tmp, " ") ||
+		if (!hbuf_puts(st->tmp, ifx_link_sep) ||
 		    !rndr_buf(st, ob, n, st->tmp, NULL))
 			return 0;
+
+		/* Format the link address. */
+
 		if (st->opts & LOWDOWN_TERM_SHORTLINK) {
 			hbuf_truncate(st->tmp);
 			if (!hbuf_shortlink
@@ -1583,52 +1667,62 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		}
 		break;
 	case LOWDOWN_IMAGE:
-		if (!rndr_buf(st, ob, n, &n->rndr_image.alt, NULL))
+		/*
+		 * This is a bit more complicated than LOWDOWN_LINK
+		 * because the image "alt" is in a buffer and not
+		 * arranged as child nodes.  Begin with the image-left
+		 * bracketing.
+		 */
+
+		hbuf_truncate(st->tmp);
+		if (!hbuf_puts(st->tmp, ifx_imgbox_left) ||
+		    !rndr_buf(st, ob, n, st->tmp, &sty_imgbox) ||
+		    !rndr_buf(st, ob, n, &n->rndr_image.alt,
+		    &sty_linkalt))
 			return 0;
-		if (n->rndr_image.alt.size) {
+
+		/* If omitting the link, right-bracket and bail. */
+
+		if ((st->opts & LOWDOWN_TERM_NOLINK) ||
+		    ((st->opts & LOWDOWN_TERM_NORELLINK) &&
+		     link_isrelative(&n->rndr_image.link))) {
 			hbuf_truncate(st->tmp);
-			if (!HBUF_PUTSL(st->tmp, " "))
-				return 0;
-			if (!rndr_buf(st, ob, n, st->tmp, NULL))
-				return 0;
-		}
-		if (st->opts & LOWDOWN_TERM_NOLINK) {
-			hbuf_truncate(st->tmp);
-			if (!hbuf_puts(st->tmp, ifx_imgbox_left))
-				return 0;
-			if (!hbuf_puts(st->tmp, ifx_imgbox_right))
-				return 0;
-			if (!rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
+			if (!hbuf_puts(st->tmp, ifx_imgbox_right) ||
+			    !rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
 				return 0;
 			break;
 		}
+
+		/* Separate between text and link address. */
+
 		hbuf_truncate(st->tmp);
-		if (!hbuf_puts(st->tmp, ifx_imgbox_left))
-			return 0;
-		if (!hbuf_puts(st->tmp, ifx_imgbox_sep))
-			return 0;
-		if (!rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
+		if (!hbuf_puts(st->tmp, ifx_imgbox_sep) ||
+		    !rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
 			return 0;
+
+		/* Format link address. */
+
 		if (st->opts & LOWDOWN_TERM_SHORTLINK) {
-			hbuf_truncate(st->tmp);
 			if (!hbuf_shortlink
 			    (st->tmp, &n->rndr_image.link))
 				return 0;
 			if (!rndr_buf(st, ob, n, st->tmp, &sty_imgurl))
 				return 0;
 		} else
-			if (!rndr_buf(st, ob, n,
-			    &n->rndr_image.link, &sty_imgurl))
+			if (!rndr_buf(st, ob, n, &n->rndr_image.link,
+			    &sty_imgurl))
 				return 0;
+
+		/* Right-bracket and end. */
+
 		hbuf_truncate(st->tmp);
-		if (!hbuf_puts(st->tmp, ifx_imgbox_right))
-			return 0;
-		if (!rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
+		if (!hbuf_puts(st->tmp, ifx_imgbox_right) ||
+		    !rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
 			return 0;
 		break;
 	case LOWDOWN_NORMAL_TEXT:
-		if (!rndr_buf(st, ob, n,
-		     &n->rndr_normal_text.text, NULL))
+		if (!rndr_buf(st, ob, n, &n->rndr_normal_text.text,
+		    NULL))
 			return 0;
 		break;
 	default:
@@ -1642,34 +1736,34 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		st->in_link = NULL;
 		break;
 	case LOWDOWN_ROOT:
-		/* Trailing block spaces. */
+		/*
+		 * If there are footnotes, begin by offsetting with
+		 * vertical space.  Then, if there's a footnote block
+		 * header, output that followed by vertical space.
+		 * Lastly, output the footnotes themselves.
+		 */
+
+		if (st->footsz && !rndr_buf_vspace(st, ob, n, 2))
+			return 0;
 		if (st->footsz) {
-			if (!rndr_buf_vspace(st, ob, n, 2))
-				return 0;
-			hbuf_truncate(st->tmp);
-			if (!hbuf_puts(st->tmp, pfx_body.text))
-				return 0;
-			if (!hbuf_puts(st->tmp, ifx_foot))
-				return 0;
-			if (!rndr_buf_literal(st, ob, n, st->tmp,
-			    &sty_foot))
+			if (!rndr_hrule(st, ob, ifx_foot, n, &sty_foot))
 				return 0;
 			if (!rndr_buf_vspace(st, ob, n, 2))
 				return 0;
-			for (i = 0; i < st->footsz; i++)
-				if (!hbuf_putb(ob, st->foots[i]) ||
-				    !HBUF_PUTSL(ob, "\n"))
-					return 0;
 		}
+		for (i = 0; i < st->footsz; i++)
+			if (!hbuf_putb(ob, st->foots[i]) ||
+			    !HBUF_PUTSL(ob, "\n"))
+				return 0;
 		if (!rndr_buf_vspace(st, ob, n, 1))
 			return 0;
+
+		/* Strip trailing newlines but for the vmargin. */
+
 		while (ob->size && ob->data[ob->size - 1] == '\n')
 			ob->size--;
 		if (!HBUF_PUTSL(ob, "\n"))
 			return 0;
-
-		/* Strip breaks but for the vmargin. */
-
 		for (i = 0; i < st->vmargin; i++)
 			if (!HBUF_PUTSL(ob, "\n"))
 				return 0;
@@ -1705,15 +1799,43 @@ lowdown_term_new(const struct lowdown_opts *opts)
 	if ((st = calloc(1, sizeof(struct term))) == NULL)
 		return NULL;
 
-	/* Give us 80 columns by default. */
-
 	if (opts != NULL) {
-		st->maxcol = opts->cols == 0 ? 80 : opts->cols;
-		st->hmargin = opts->hmargin;
-		st->vmargin = opts->vmargin;
+		/*
+		 * Compute the width of the content pre-padding.  If
+		 * zero, limit to 80 or the number of terminal columns.
+		 * Otherwise, truncate to the number of columns.
+		 */
+
+		if (opts->term.width == 0) {
+			if ((st->width = opts->term.cols) > 80)
+				st->width = 80;
+		} else if (opts->term.width > opts->term.cols) {
+			st->width = opts->term.cols;
+		} else
+			st->width = opts->term.width;
+
+		/*
+		 * Compute the horizontal margin: either as given or, if
+		 * centred, computed from the content width.
+		 */
+
+		if (opts->term.centre && st->width < opts->term.cols)
+			st->hmargin = (opts->term.cols - st->width) / 2;
+		else
+			st->hmargin = opts->term.hmargin;
+
+		st->hpadding = opts->term.hpadding;
+		st->vmargin = opts->term.vmargin;
 		st->opts = opts->oflags;
-	} else
-		st->maxcol = 80;
+	} else {
+		st->width = 80;
+		st->hpadding = 4;
+	}
+
+	if (st->hpadding >= st->width)
+		st->width = 1;
+	else
+		st->width -= st->hpadding;
 
 	if ((st->tmp = hbuf_new(32)) == NULL) {
 		free(st);
@@ -1726,7 +1848,7 @@ void
 lowdown_term_free(void *arg)
 {
 	struct term	*st = arg;
-	
+
 	if (st == NULL)
 		return;
 
